@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,23 +25,62 @@ should attempt to make run time detections.
 
 import __future__
 
+import ctypes
 import os
 import re
 import sys
 
-from nuitka.__past__ import WindowsError  # pylint: disable=I0021,redefined-builtin
-
 
 def getSupportedPythonVersions():
     """Officially supported Python versions for Nuitka."""
 
-    return ("2.6", "2.7", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9")
+    return ("2.6", "2.7", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9", "3.10")
+
+
+def getNotYetSupportedPythonVersions():
+    """Versions known to not work at all (yet)."""
+    return ()
 
 
 def getPartiallySupportedPythonVersions():
     """Partially supported Python versions for Nuitka."""
 
-    return ("3.10",)
+    return ("3.11",)
+
+
+def getZstandardSupportingVersions():
+    result = getSupportedPythonVersions() + getPartiallySupportedPythonVersions()
+
+    # This will crash if we remove versions, but it is more likely to work
+    # with newly supported versions, and to list the ones not supported by
+    # zstandard.
+    result = tuple(
+        version for version in result if version not in ("2.6", "2.7", "3.3", "3.4")
+    )
+
+    return result
+
+
+def getTestExecutionPythonVersions():
+    return (
+        getSupportedPythonVersions()
+        + getPartiallySupportedPythonVersions()
+        + getNotYetSupportedPythonVersions()
+    )
+
+
+# Make somewhat sure we keep these ones consistent
+assert len(
+    set(
+        getPartiallySupportedPythonVersions()
+        + getNotYetSupportedPythonVersions()
+        + getSupportedPythonVersions()
+    )
+) == len(
+    getPartiallySupportedPythonVersions()
+    + getNotYetSupportedPythonVersions()
+    + getSupportedPythonVersions()
+)
 
 
 def getSupportedPythonVersionStr():
@@ -109,6 +148,21 @@ def getComplexCallSequenceErrorTemplate():
     return getComplexCallSequenceErrorTemplate.result
 
 
+def getUnboundLocalErrorErrorTemplate():
+    if not hasattr(getUnboundLocalErrorErrorTemplate, "result"):
+        try:
+            # We are doing this on purpose, to get the exception.
+            # pylint: disable=undefined-variable
+            del _f
+        except UnboundLocalError as e:
+            result = e.args[0].replace("_f", "%s")
+            getUnboundLocalErrorErrorTemplate.result = result
+        else:
+            sys.exit("Error, cannot detect expected error message.")
+
+    return getUnboundLocalErrorErrorTemplate.result
+
+
 _needs_set_literal_reverse_insertion = None
 
 
@@ -136,47 +190,29 @@ def needsDuplicateArgumentColOffset():
 
 
 def getRunningPythonDLLPath():
-    import ctypes.wintypes
-
-    MAX_PATH = 4096
-    buf = ctypes.create_unicode_buffer(MAX_PATH)
-
-    GetModuleFileName = ctypes.windll.kernel32.GetModuleFileNameW
-    GetModuleFileName.argtypes = (
-        ctypes.wintypes.HANDLE,
-        ctypes.wintypes.LPWSTR,
-        ctypes.wintypes.DWORD,
+    from nuitka.utils.SharedLibraries import (
+        getWindowsRunningProcessModuleFilename,
     )
-    GetModuleFileName.restype = ctypes.wintypes.DWORD
 
     # We trust ctypes internals here, pylint: disable=protected-access
-    res = GetModuleFileName(ctypes.pythonapi._handle, buf, MAX_PATH)
-    if res == 0:
-        # Windows only code, pylint: disable=I0021,undefined-variable
-        raise WindowsError(
-            ctypes.GetLastError(), ctypes.FormatError(ctypes.GetLastError())
-        )
-
-    dll_path = os.path.normcase(buf.value)
-    assert os.path.exists(dll_path), dll_path
-
-    return dll_path
+    # spell-checker: ignore pythonapi
+    return getWindowsRunningProcessModuleFilename(ctypes.pythonapi._handle)
 
 
 def getTargetPythonDLLPath():
     dll_path = getRunningPythonDLLPath()
 
-    from nuitka.Options import isPythonDebug
+    from nuitka.Options import shallUsePythonDebug
 
     if dll_path.endswith("_d.dll"):
-        if not isPythonDebug():
+        if not shallUsePythonDebug():
             dll_path = dll_path[:-6] + ".dll"
 
         if not os.path.exists(dll_path):
             sys.exit("Error, cannot switch to non-debug Python, not installed.")
 
     else:
-        if isPythonDebug():
+        if shallUsePythonDebug():
             dll_path = dll_path[:-4] + "_d.dll"
 
         if not os.path.exists(dll_path):
@@ -186,10 +222,8 @@ def getTargetPythonDLLPath():
 
 
 def isStaticallyLinkedPython():
-    # On Windows, there is no way to detect this from syconfig.
+    # On Windows, there is no way to detect this from sysconfig.
     if os.name == "nt":
-        import ctypes
-
         return ctypes.pythonapi is None
 
     try:
@@ -209,9 +243,10 @@ def getPythonABI():
         abiflags = sys.abiflags
 
         # Cyclic dependency here.
-        from nuitka.Options import isPythonDebug
+        from nuitka.Options import shallUsePythonDebug
 
-        if isPythonDebug() or hasattr(sys, "getobjects"):
+        # spell-checker: ignore getobjects
+        if shallUsePythonDebug() or hasattr(sys, "getobjects"):
             if not abiflags.startswith("d"):
                 abiflags = "d" + abiflags
     else:
@@ -239,7 +274,6 @@ def getSystemPrefixPath():
 
     global _the_sys_prefix  # Cached result, pylint: disable=global-statement
     if _the_sys_prefix is None:
-
         sys_prefix = getattr(
             sys, "real_prefix", getattr(sys, "base_prefix", sys.prefix)
         )
@@ -280,6 +314,12 @@ def getSystemPrefixPath():
 
             sys_prefix = os.path.normpath(os.path.join(python_binary, "..", ".."))
 
+        # Resolve symlinks on Windows manually.
+        if os.name == "nt":
+            from nuitka.utils.FileOperations import getDirectoryRealPath
+
+            sys_prefix = getDirectoryRealPath(sys_prefix)
+
         _the_sys_prefix = sys_prefix
 
     return _the_sys_prefix
@@ -315,3 +355,53 @@ def getImportlibSubPackages():
             result.append(module_info[1])
 
     return result
+
+
+def isDebugPython():
+    """Is this a debug build of Python."""
+    return hasattr(sys, "gettotalrefcount")
+
+
+def _getFloatDigitBoundaryValue():
+    if python_version < 0x270:
+        bits_per_digit = 15
+    elif python_version < 0x300:
+        bits_per_digit = sys.long_info.bits_per_digit
+    else:
+        bits_per_digit = sys.int_info.bits_per_digit
+
+    return (2**bits_per_digit) - 1
+
+
+_float_digit_boundary = _getFloatDigitBoundaryValue()
+
+
+def isPythonValidDigitValue(value):
+    """Does the given value fit into a float digit.
+
+    Note: Digits in long objects do not use 2-complement, but a boolean sign.
+    """
+
+    return -_float_digit_boundary <= value <= _float_digit_boundary
+
+
+sizeof_clong = ctypes.sizeof(ctypes.c_long)
+
+# TODO: We could be more aggressive here, there are issues with using full
+# values, in some contexts, but that can probably be sorted out.
+_max_signed_long = 2 ** (sizeof_clong * 7) - 1
+_min_signed_long = -(2 ** (sizeof_clong * 7))
+
+# Used by data composer to write Python long values.
+sizeof_clonglong = ctypes.sizeof(ctypes.c_longlong)
+
+_max_signed_longlong = 2 ** (sizeof_clonglong * 8 - 1) - 1
+_min_signed_longlong = -(2 ** (sizeof_clonglong * 8 - 1))
+
+
+def isPythonValidCLongValue(value):
+    return _min_signed_long <= value <= _max_signed_long
+
+
+def isPythonValidCLongLongValue(value):
+    return _min_signed_longlong <= value <= _max_signed_longlong
