@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,11 +23,11 @@ We use depends.exe to investigate needed DLLs of Python DLLs.
 
 import os
 
-from nuitka.containers.oset import OrderedSet
+from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.Options import assumeYesForDownloads
 from nuitka.Tracing import inclusion_logger
 from nuitka.utils.Download import getCachedDownload
-from nuitka.utils.Execution import executeProcess
+from nuitka.utils.Execution import executeProcess, withEnvironmentVarOverridden
 from nuitka.utils.FileOperations import (
     deleteFile,
     getExternalUsePath,
@@ -35,6 +35,7 @@ from nuitka.utils.FileOperations import (
     putTextFileContents,
     withFileLock,
 )
+from nuitka.utils.SharedLibraries import getWindowsRunningProcessDLLPaths
 from nuitka.utils.Utils import getArchitecture
 
 
@@ -63,6 +64,29 @@ to analyze the dependencies of Python extension modules.""",
     )
 
 
+def _attemptToFindNotFoundDLL(dll_filename):
+    """Some heuristics and tricks to find DLLs that dependency walker did not find."""
+
+    # Lets attempt to find it on currently loaded DLLs, this typically should
+    # find the Python DLL.
+    currently_loaded_dlls = getWindowsRunningProcessDLLPaths()
+
+    if dll_filename in currently_loaded_dlls:
+        return currently_loaded_dlls[dll_filename]
+
+    dll_filename = os.path.join(
+        os.environ["SYSTEMROOT"],
+        "SysWOW64" if getArchitecture() == "x86_64" else "System32",
+        dll_filename,
+    )
+    dll_filename = os.path.normcase(dll_filename)
+
+    if os.path.exists(dll_filename):
+        return dll_filename
+
+    return None
+
+
 def _parseDependsExeOutput2(lines):
     result = OrderedSet()
 
@@ -91,19 +115,20 @@ def _parseDependsExeOutput2(lines):
         if "E" in line[: line.find("]")]:
             continue
 
-        # Skip missing DLLs, apparently not needed anyway.
+        # Skip missing DLLs, apparently not needed anyway, but we can still
+        # try a few tricks
         if "?" in line[: line.find("]")]:
-            # One exception are PythonXY.DLL
+            # One exception are "PythonXY.DLL", we try to find them from Windows folder.
             if dll_filename.startswith("python") and dll_filename.endswith(".dll"):
-                dll_filename = os.path.join(
-                    os.environ["SYSTEMROOT"],
-                    "SysWOW64" if getArchitecture() == "x86_64" else "System32",
-                    dll_filename,
-                )
-                dll_filename = os.path.normcase(dll_filename)
+                dll_filename = _attemptToFindNotFoundDLL(dll_filename)
+
+                if dll_filename is None:
+                    continue
             else:
                 continue
 
+        assert os.path.basename(dll_filename) != "kernel32.dll"
+
         dll_filename = os.path.abspath(dll_filename)
 
         dll_name = os.path.basename(dll_filename)
@@ -129,9 +154,11 @@ def parseDependsExeOutput(filename):
     return _parseDependsExeOutput2(getFileContentByLine(filename, encoding="latin1"))
 
 
-def detectDLLsWithDependencyWalker(binary_filename, scan_dirs):
-    dwp_filename = binary_filename + ".dwp"
-    output_filename = binary_filename + ".depends"
+def detectDLLsWithDependencyWalker(binary_filename, source_dir, scan_dirs):
+    dwp_filename = os.path.join(source_dir, os.path.basename(binary_filename) + ".dwp")
+    output_filename = os.path.join(
+        source_dir, os.path.basename(binary_filename) + ".depends"
+    )
 
     # User query should only happen once if at all.
     with withFileLock(
@@ -146,7 +173,6 @@ def detectDLLsWithDependencyWalker(binary_filename, scan_dirs):
             dwp_filename,
             contents="""\
 %(scan_dirs)s
-SxS
 """
             % {
                 "scan_dirs": "\n".join(
@@ -159,19 +185,20 @@ SxS
     # TODO: At least exit code should be checked, output goes to a filename,
     # but errors might be interesting potentially.
 
-    _stdout, _stderr, _exit_code = executeProcess(
-        command=(
-            depends_exe,
-            "-c",
-            "-ot%s" % output_filename,
-            "-d:%s" % dwp_filename,
-            "-f1",
-            "-pa1",
-            "-ps1",
-            binary_filename,
-        ),
-        external_cwd=True,
-    )
+    with withEnvironmentVarOverridden("PATH", ""):
+        _stdout, _stderr, _exit_code = executeProcess(
+            command=(
+                depends_exe,
+                "-c",
+                "-ot%s" % output_filename,
+                "-d:%s" % dwp_filename,
+                "-f1",
+                "-pa1",
+                "-ps1",
+                binary_filename,
+            ),
+            external_cwd=True,
+        )
 
     if not os.path.exists(output_filename):
         inclusion_logger.sysexit(
