@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,17 +23,19 @@ for some types, so it is still useful. Also calls to it can be accelerated
 slightly.
 """
 
-from .ExpressionBases import ExpressionChildHavingBase
+from .ChildrenHavingMixins import ChildHavingValueMixin
+from .ExpressionBases import ExpressionBase
 
 
-class ExpressionBuiltinHash(ExpressionChildHavingBase):
+class ExpressionBuiltinHash(ChildHavingValueMixin, ExpressionBase):
     kind = "EXPRESSION_BUILTIN_HASH"
 
-    named_child = "value"
-    getValue = ExpressionChildHavingBase.childGetter("value")
+    named_children = ("value",)
 
     def __init__(self, value, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=value, source_ref=source_ref)
+        ChildHavingValueMixin.__init__(self, value=value)
+
+        ExpressionBase.__init__(self, source_ref)
 
     def computeExpression(self, trace_collection):
         value = self.subnode_value
