@@ -38,10 +38,12 @@ from .NodeMakingHelpers import (
     wrapExpressionWithNodeSideEffects,
 )
 from .shapes.BuiltinTypeShapes import (
+    tshape_bool,
     tshape_bytes,
     tshape_dict,
     tshape_list,
     tshape_str,
+    tshape_type,
     tshape_unicode,
 )
 from .shapes.StandardShapes import tshape_unknown
@@ -88,6 +90,11 @@ class ExpressionBase(NodeBase):
 
         return False, None
 
+    @staticmethod
+    def isMappingWithConstantStringKeys():
+        """Is this a mapping with constant string keys. Used for call optimization."""
+        return False
+
     @staticmethod
     def isKnownToBeIterable(count):
         """Can be iterated at all (count is None) or exactly count times.
@@ -864,6 +871,11 @@ class ExpressionBase(NodeBase):
     def hasShapeTrustedAttributes(self):
         return self.getTypeShape().hasShapeTrustedAttributes()
 
+    def hasShapeTypeExact(self):
+        """Does a node have exactly a 'type' shape."""
+
+        return self.getTypeShape() is tshape_type
+
     def hasShapeListExact(self):
         """Does a node have exactly a list shape."""
 
@@ -896,16 +908,15 @@ class ExpressionBase(NodeBase):
         """Does an expression have exactly a bytes shape."""
         return self.getTypeShape() is tshape_bytes
 
+    def hasShapeBoolExact(self):
+        """Does an expression have exactly a bool shape."""
+        return self.getTypeShape() is tshape_bool
+
     @staticmethod
     def hasVeryTrustedValue():
         """Trust that value will not be overwritten from the outside."""
         return False
 
-    @staticmethod
-    def getUsedModules():
-        """Nodes should indicate modules that they (attempted to) use here."""
-        return ()
-
 
 class ExpressionNoSideEffectsMixin(object):
     __slots__ = ()
