@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,6 +26,7 @@ make others possible.
 import inspect
 
 from nuitka import ModuleRegistry, Options, Variables
+from nuitka.importing.Importing import addExtraSysPaths
 from nuitka.importing.Recursion import considerUsedModules
 from nuitka.plugins.Plugins import Plugins
 from nuitka.Progress import (
@@ -33,16 +34,9 @@ from nuitka.Progress import (
     reportProgressBar,
     setupProgressBar,
 )
-from nuitka.Tracing import (
-    general,
-    memory_logger,
-    optimization_logger,
-    progress_logger,
-)
-from nuitka.utils.MemoryUsage import (
-    MemoryWatch,
-    getHumanReadableProcessMemoryUsage,
-)
+from nuitka.Tracing import general, optimization_logger, progress_logger
+from nuitka.utils.MemoryUsage import MemoryWatch, reportMemoryUsage
+from nuitka.utils.Timing import TimerReport
 
 from . import Graphs
 from .BytecodeDemotion import demoteCompiledModuleToBytecode
@@ -139,11 +133,8 @@ def optimizeCompiledPythonModule(module):
         touched = True
 
     if Options.isShowProgress() and Options.isShowMemory():
-        memory_watch.finish()
-
-        memory_logger.info(
-            "Memory usage changed during optimization of '%s': %s"
-            % (module.getFullName(), memory_watch.asStr())
+        memory_watch.finish(
+            "Memory usage changed during optimization of '%s'" % (module.getFullName())
         )
 
     considerUsedModules(module=module, signal_change=signalChange)
@@ -180,6 +171,8 @@ def optimizeModule(module):
     global tag_set
     tag_set = TagSet()
 
+    addExtraSysPaths(Plugins.getModuleSysPathAdditions(module.getFullName()))
+
     if module.isPythonExtensionModule():
         optimizeExtensionModule(module)
         changed = False
@@ -235,12 +228,16 @@ after that.""".format(
     )
 
     if Options.isShowProgress() and Options.isShowMemory():
-        output = "Memory usage {memory}:".format(
-            memory=getHumanReadableProcessMemoryUsage()
+        reportMemoryUsage(
+            "optimization/%d/%s" % (pass_count, current_module.getFullName()),
+            (
+                "Total memory usage before optimizing module '%s'"
+                % current_module.getFullName()
+            )
+            if Options.isShowProgress() or Options.isShowMemory()
+            else None,
         )
 
-        memory_logger.info(output)
-
 
 def _traceProgressModuleEnd(current_module):
     reportProgressBar(
@@ -288,7 +285,20 @@ def makeOptimizationPass():
 
         _traceProgressModuleStart(current_module)
 
-        changed = optimizeModule(current_module)
+        module_name = current_module.getFullName()
+
+        with TimerReport(
+            message="Optimizing %s" % module_name, decider=False
+        ) as module_timer:
+            changed = optimizeModule(current_module)
+
+            # module_timer=module_time_report.getTimer()
+
+        ModuleRegistry.addModuleOptimizationTimeInformation(
+            module_name=module_name,
+            pass_number=pass_count,
+            time_used=module_timer.getDelta(),
+        )
 
         _traceProgressModuleEnd(current_module)
 
@@ -310,14 +320,12 @@ def makeOptimizationPass():
                 unused_function.trace_collection = None
                 unused_function.finalize()
 
-            used_functions = tuple(
+            current_module.subnode_functions = tuple(
                 function
                 for function in current_module.subnode_functions
                 if function in current_module.getUsedFunctions()
             )
 
-            current_module.setChild("functions", used_functions)
-
     _endProgress()
 
     return finished
