@@ -207,6 +207,10 @@ class ValueTraceBase(object):
     def hasShapeTupleExact():
         return False
 
+    @staticmethod
+    def hasShapeBoolExact():
+        return False
+
     @staticmethod
     def getTruthValue():
         return None
@@ -241,6 +245,17 @@ class ValueTraceBase(object):
         """Node to use for iteration decisions."""
         return None
 
+    @staticmethod
+    def getDictInValue(key):
+        """Value to use for dict in decisions."""
+
+        # virtual method, pylint: disable=unused-argument
+        return None
+
+    @staticmethod
+    def inhibitsClassScopeForwardPropagation():
+        return True
+
 
 class ValueTraceUnassignedBase(ValueTraceBase):
     __slots__ = ()
@@ -284,6 +299,9 @@ class ValueTraceUninitialized(ValueTraceUnassignedBase):
     def isTraceThatNeedsEscape():
         return False
 
+    def inhibitsClassScopeForwardPropagation(self):
+        return False
+
 
 class ValueTraceDeleted(ValueTraceUnassignedBase):
     """Trace caused by a deletion."""
@@ -540,6 +558,9 @@ class ValueTraceAssign(ValueTraceBase):
     def hasShapeUnicodeExact(self):
         return self.assign_node.subnode_source.hasShapeUnicodeExact()
 
+    def hasShapeBoolExact(self):
+        return self.assign_node.subnode_source.hasShapeBoolExact()
+
     def getTruthValue(self):
         return self.assign_node.subnode_source.getTruthValue()
 
@@ -567,6 +588,13 @@ class ValueTraceAssign(ValueTraceBase):
     def getIterationSourceNode(self):
         return self.assign_node.subnode_source
 
+    def getDictInValue(self, key):
+        """Value to use for dict in decisions."""
+        return self.assign_node.subnode_source.getExpressionDictInConstant(key)
+
+    def inhibitsClassScopeForwardPropagation(self):
+        return self.assign_node.subnode_source.mayHaveSideEffects()
+
 
 class ValueTraceAssignUnescapable(ValueTraceAssign):
     @staticmethod
