@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -35,16 +35,6 @@ def visitTree(tree, visitor):
     visitor.onLeaveNode(tree)
 
 
-def visitFunction(function, visitor):
-    visitor.onEnterNode(function)
-    visitor.onLeaveNode(function)
-
-
-def visitModule(module, visitor):
-    visitor.onEnterNode(module)
-    visitor.onLeaveNode(module)
-
-
 class VisitorNoopMixin(object):
     def onEnterNode(self, node):
         """Overloaded for operation before the node children were done."""
